/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lib.h"
#include "imap-util.h"

const char *imap_write_flags(MailFlags flags)
{
	const char *str;

	str = t_strconcat((flags & MAIL_ANSWERED) ? " \\Answered" : "",
			  (flags & MAIL_FLAGGED) ? " \\Flagged" : "",
			  (flags & MAIL_DELETED) ? " \\Deleted" : "",
			  (flags & MAIL_SEEN) ? " \\Seen" : "",
			  (flags & MAIL_DRAFT) ? " \\Draft" : "",
			  (flags & MAIL_RECENT)  ? " \\Recent" : "",
			  NULL);

	return *str == '\0' ? "" : str+1;;
}

const char *imap_escape(const char *str)
{
	char *ret, *p;
	int i, esc;

	/* get length of string and number of chars to escape */
	esc = 0;
	for (i = 0; str[i] != '\0'; i++) {
		if (IS_ESCAPED_CHAR(str[i]))
			esc++;
	}

	if (esc == 0)
		return str;

	/* escape them */
	p = ret = t_malloc(i + esc + 1);
	for (; *str != '\0'; str++) {
		if (IS_ESCAPED_CHAR(str[i]))
			*p++ = '\\';
		*p++ = *str;
	}
	*p = '\0';
	return ret;
}

int imap_mailbox_mask_match(const char *name, const char *mask,
			    char hierarchy_separator)
{
	/* FIXME: do this properly */
	while (*name != '\0' && *mask != '\0') {
		if (*mask == '*' || *mask == '%')
			return TRUE;

		if (*name != *mask)
			return FALSE;
		name++; mask++;
	}
	while (*mask == '*') mask++;
	return *name == '\0' && *mask == '\0';
}
