/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lib.h"
#include "iobuffer.h"
#include "index-storage.h"
#include "mail-messageset.h"

#include <unistd.h>

static int copy_func(MailIndex *index, MailIndexRecord *rec,
		     unsigned int seq, void *user_data)
{
	IndexMailbox *dest = user_data;
	IOBuffer *buf;
	off_t offset;
	size_t size;
	int fd, failed;

	fd = index->open_mail(index, rec, &offset, &size);
	if (fd == -1)
		return FALSE;

	/* save it in destination mailbox */
	buf = io_buffer_create_file(fd, default_pool, 4096);
	failed = !dest->box.save(&dest->box,
				 rec->msg_flags & MAIL_SYSTEM_FLAGS_MASK,
				 rec->internal_date, buf, size);

	(void)close(fd);
	return !failed;
}

int index_storage_copy(Mailbox *box, Mailbox *destbox,
		       const char *messageset, int uidset)
{
	IndexMailbox *ibox = (IndexMailbox *) box;
	int failed;

	if (destbox->readonly) {
		mail_storage_set_error(box->storage,
				       "Destination mailbox is read-only");
		return FALSE;
	}

	if (!ibox->index->set_lock(ibox->index, MAIL_LOCK_SHARED))
		return mail_storage_set_index_error(ibox);

	if (uidset) {
		failed = mail_index_uidset_foreach(ibox->index, messageset,
						   ibox->synced_messages_count,
						   copy_func, destbox) <= 0;
	} else {
		failed = mail_index_messageset_foreach(ibox->index, messageset,
			ibox->synced_messages_count, copy_func, destbox) <= 0;
	}

	if (!ibox->index->set_lock(ibox->index, MAIL_LOCK_UNLOCK) || failed)
		return mail_storage_set_index_error(ibox);

	return TRUE;
}

