#ifndef __DYNARRAY_H
#define __DYNARRAY_H

typedef void (*PForeachFunc)(void *data, void *user_data);
typedef int (*PFindFunc)(void *data, void *user_data);

struct _PArray {
	int last;
	void **array;
};

typedef struct _PArray *PArray;

/* Unsorted array of pointers. Note that the index of
   allocated items may change. */
PArray parray_create(Pool pool, int initial_items);
void parray_destroy(PArray array);

#ifdef POOL_CHECK_LEAKS
#  define parray_destroy_clean(array) parray_destroy(array)
#else
#  define parray_destroy_clean(array)
#endif

int parray_alloc(PArray array, void *data);
void parray_free(PArray array, int index);
void parray_clear(PArray array);

void parray_foreach(PArray array, PForeachFunc func, void *user_data);
int parray_find(PArray array, void *data);
int parray_find_custom(PArray array, PFindFunc func, void *user_data);
void *parray_find_custom_data(PArray array, PFindFunc func, void *user_data);

/* While array is frozen, parray_free() just marks the position as NULL.
   This is useful when removing data from array in for (..) loop.
   Remember to check if data is NULL before accessing it. */
void parray_freeze(PArray array);
void parray_thaw(PArray array);

int parray_size(PArray);

#define parray_is_empty(_array) ((_array) == NULL || (_array)->last == 0)
#define parray_index(_array, index) \
	((index) >= 0 && (index) < (_array)->last ? \
		(_array)->array[index] : NULL)

#endif
