/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lib.h"

#include <stdlib.h>
#include <unistd.h>
#include <grp.h>

void restrict_access_set_env(const char *user, uid_t uid, gid_t gid,
			     const char *chroot_dir)
{
	if (user != NULL && *user != '\0')
		putenv((char *) t_strconcat("USER=", user, NULL));
	if (chroot_dir != NULL && *chroot_dir != '\0')
		putenv((char *) t_strconcat("CHROOT=", chroot_dir, NULL));

	putenv((char *) t_strdup_printf("SETUID=%ld", (long) uid));
	putenv((char *) t_strdup_printf("SETGID=%ld", (long) gid));
}

void restrict_access_by_env(void)
{
	const char *env;
	gid_t gid;
	uid_t uid;

	/* chrooting */
	env = getenv("CHROOT");
	if (env != NULL) {
		if (chroot(env) != 0)
			i_fatal("chroot(%s) failed: %s", env, strerror(errno));

		if (chdir("/") != 0)
			i_fatal("chdir(/) failed: %s", strerror(errno));
	}

	/* groups - the getgid() checks are just so we don't fail if we're
	   not running as root and try to just use our own GID. */
	env = getenv("SETGID");
	gid = env == NULL ? 0 : (gid_t) atol(env);
	if (gid != 0 && (gid != getgid() || gid != getegid())) {
		if (setgid(gid) != 0) {
			i_fatal("setgid(%ld) failed: %s",
				(long) gid, strerror(errno));
		}

		env = getenv("USER");
		if (env == NULL) {
			/* user not known, use only this one group */
			(void)setgroups(1, &gid);
		} else {
			if (initgroups(env, gid) != 0) {
				i_fatal("initgroups(%s, %ld) failed: %s",
					env, (long) gid, strerror(errno));
			}
		}
	}

	/* uid last */
	env = getenv("SETUID");
	uid = env == NULL ? 0 : (uid_t) atol(env);
	if (uid != 0) {
		if (setuid(uid) != 0) {
			i_fatal("setuid(%ld) failed: %s",
				(long) uid, strerror(errno));
		}

		/* just extra verification */
#ifdef HAVE_SETREUID
		if (setreuid(-1, 0) == 0)
#else
		if (setuid(0) == 0)
#endif
			i_fatal("We couldn't drop root privileges");
	}
}
